// This demonstrates that open / truncate / close on a sparse file on
// Windows causes the next open to take 600ms

package main

import (
	"fmt"
	"log"
	"os"
	"time"
)

// open file_name for read/write truncating it to its current size
//
// if truncate is set it truncates the file to its existing size after
// opening
func open(file_name string, truncate bool) *os.File {
	t1 := time.Now()
	f, err := os.OpenFile(file_name, os.O_RDWR, 0600)
	fmt.Printf("OpenFile took %v\n", time.Since(t1))
	if err != nil {
		log.Fatal(err)
	}
	return f
}

// truncate the file to its current size
//
// this should be a no-op
func truncate(f *os.File) {
	fi, err := f.Stat()
	if err != nil {
		log.Fatal(fi, err)
	}
	t2 := time.Now()
	err = f.Truncate(fi.Size())
	fmt.Printf("Truncate took %v\n", time.Since(t2))
	if err != nil {
		log.Fatal(err)
	}
}

// close fd reporting error
func close(f *os.File) {
	t1 := time.Now()
	err := f.Close()
	fmt.Printf("Close took %v\n", time.Since(t1))
	if err != nil {
		log.Fatal(err)
	}
}

func main() {
	if len(os.Args) < 2 {
		log.Fatalf("Syntax %s <sparse-file>", os.Args[0])
	}

	fileName := os.Args[1]

	f := open(fileName, false)
	close(f)

	f = open(fileName, true)
	truncate(f)
	close(f)

	f = open(fileName, false)
	close(f)

	f = open(fileName, false)
	truncate(f)
	close(f)

	f = open(fileName, false)
	close(f)
}
